#ifndef __LINUX_NETLINK_H
#define __LINUX_NETLINK_H

#include <stdint.h>
#include <sys/socket.h>                 /* for sa_family_t */

#define NETLINK_ROUTE           0       /* Routing/device hook				*/
#define NETLINK_UNUSED          1       /* Unused number				*/
#define NETLINK_USERSOCK        2       /* Reserved for user mode socket protocols      */
#define NETLINK_FIREWALL        3       /* Firewalling hook				*/
#define NETLINK_INET_DIAG       4       /* INET socket monitoring			*/
#define NETLINK_NFLOG           5       /* netfilter/iptables ULOG */
#define NETLINK_XFRM            6       /* ipsec */
#define NETLINK_SELINUX         7       /* SELinux event notifications */
#define NETLINK_ISCSI           8       /* Open-iSCSI */
#define NETLINK_AUDIT           9       /* auditing */
#define NETLINK_FIB_LOOKUP      10
#define NETLINK_CONNECTOR       11
#define NETLINK_NETFILTER       12      /* netfilter subsystem */
#define NETLINK_IP6_FW          13
#define NETLINK_DNRTMSG         14      /* DECnet routing messages */
#define NETLINK_KOBJECT_UEVENT  15      /* Kernel messages to userspace */
#define NETLINK_GENERIC         16
/* leave room for NETLINK_DM (DM Events) */
#define NETLINK_SCSITRANSPORT   18      /* SCSI Transports */
#define NETLINK_ECRYPTFS        19
#define NETLINK_RDMA            20

#define MAX_LINKS 32

struct sockaddr_nl {
	sa_family_t nl_family;          /* AF_NETLINK	*/
	unsigned short nl_pad;          /* zero		*/
	uint32_t nl_pid;                /* process pid	*/
	uint32_t nl_groups;             /* multicast groups mask */
};

struct nlmsghdr {
	uint32_t nlmsg_len;             /* Length of message including header */
	uint16_t nlmsg_type;            /* Message content */
	uint16_t nlmsg_flags;           /* Additional flags */
	uint32_t nlmsg_seq;             /* Sequence number */
	uint32_t nlmsg_pid;             /* Sending process PID */
};

/* Flags values */

#define NLM_F_REQUEST           1       /* It is request message.       */
#define NLM_F_MULTI             2       /* Multipart message, terminated by NLMSG_DONE */
#define NLM_F_ACK               4       /* Reply with ack, with zero or error code */
#define NLM_F_ECHO              8       /* Echo this request            */

/* Modifiers to GET request */
#define NLM_F_ROOT      0x100   /* specify tree	root	*/
#define NLM_F_MATCH     0x200   /* return all matching	*/
#define NLM_F_ATOMIC    0x400   /* atomic GET		*/
#define NLM_F_DUMP      (NLM_F_ROOT | NLM_F_MATCH)

/* Modifiers to NEW request */
#define NLM_F_REPLACE   0x100   /* Override existing		*/
#define NLM_F_EXCL      0x200   /* Do not touch, if it exists	*/
#define NLM_F_CREATE    0x400   /* Create, if it does not exist	*/
#define NLM_F_APPEND    0x800   /* Add to end of list		*/

/*
   4.4BSD ADD		NLM_F_CREATE|NLM_F_EXCL
   4.4BSD CHANGE	NLM_F_REPLACE

   True CHANGE		NLM_F_CREATE|NLM_F_REPLACE
   Append		NLM_F_CREATE
   Check		NLM_F_EXCL
 */

#define NLMSG_ALIGNTO   4
#define NLMSG_ALIGN(len) ( ((len) + NLMSG_ALIGNTO - 1) & ~(NLMSG_ALIGNTO - 1) )
#define NLMSG_LENGTH(len) ((len) + NLMSG_ALIGN(sizeof(struct nlmsghdr)))
#define NLMSG_SPACE(len) NLMSG_ALIGN(NLMSG_LENGTH(len))
#define NLMSG_DATA(nlh)  ((void*)(((char*)(nlh)) + NLMSG_LENGTH(0)))

#define NLMSG_NEXT(nlh, len) \
	((len) -= NLMSG_ALIGN((nlh)->nlmsg_len), \
	 (struct nlmsghdr*)(((char*)(nlh)) + NLMSG_ALIGN((nlh)->nlmsg_len)))

#define NLMSG_OK(nlh, len) \
	((len) > 0 && \
	 (nlh)->nlmsg_len >= sizeof(struct nlmsghdr) && \
	 (nlh)->nlmsg_len <= (len))

#define NLMSG_PAYLOAD(nlh, len) ((nlh)->nlmsg_len - NLMSG_SPACE((len)))

#define NLMSG_NOOP              0x1     /* Nothing.		*/
#define NLMSG_ERROR             0x2     /* Error		*/
#define NLMSG_DONE              0x3     /* End of a dump	*/
#define NLMSG_OVERRUN           0x4     /* Data lost		*/

struct nlmsgerr {
	int error;
	struct nlmsghdr msg;
};

#define NET_MAJOR 36            /* Major 36 is reserved for networking                                          */
#endif  /* __LINUX_NETLINK_H */
